/*******************************************************************************
* RGB  - Function Set for RGB Example
* ------------------------------------------------------------------------------
* Reads color value from picture which is stored in SD card
*   and sets RGB LED in the same color
*******************************************************************************/
#include "mikromedia_5_TIVA_objects.h"
#include "Init_routines.h"

// external globals
extern unsigned int Xcoord, Ycoord;

// globals
char RGB_Image_Loaded = 0;
unsigned int * const RGB_Color_Image_SRAM_ptr = 0x10100000;


// color struct
typedef struct Colors {
  char Red;
  char Green;
  char Blue;
  unsigned int color_16bit;
} TColors;

// static
static int xcoo, ycoo, color;
static unsigned int pwm_period1, pwm_period2, pwm_period3;
static TColors MyColors;

/*******************************************************************************
* Function RGB_Init()
* ------------------------------------------------------------------------------
* Overview: Function Initializes PWMs and RGB connection pins
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void RGB_Init() {
  pwm_period1 = PWM_T1CCP0_Init(5000, &_GPIO_MODULE_T1CCP0_D2_AHB);       // R
  pwm_period2 = PWM_T1CCP1_Init(5000, &_GPIO_MODULE_T1CCP1_D3_AHB);       // G
  pwm_period3 = PWM_T3CCP1_Init(5000, &_GPIO_MODULE_T3CCP1_D5_AHB);       // B

  PWM_T1CCP0_Set_Duty(1, _PWM_INVERTED_DISABLE);
  PWM_T1CCP1_Set_Duty(1, _PWM_INVERTED_DISABLE);
  PWM_T3CCP1_Set_Duty(1, _PWM_INVERTED_DISABLE);
  
  PWM_T1CCP0_Start();
  PWM_T1CCP1_Start();
  PWM_T3CCP1_Start();
  
  if (RGB_Image_Loaded){
    TFTIntern_Image(0, 60, (char *)RGB_Color_Image_SRAM_ptr, 1);
  }
  else
    if (Init_FAT()){                                          // displays background picture
      CopyImage_FromResources_ToSRAM(rgb_bitmap_bmp, RGB_Color_Image_SRAM_ptr);
      TFTIntern_Image(0, 60, (char *)RGB_Color_Image_SRAM_ptr, 1);
      RGB_Image_Loaded = 1;
    }
}

/*******************************************************************************
* Function GetColor_FromSD(unsigned long Img, unsigned int xc, unsigned int yc)
* ------------------------------------------------------------------------------
* Overview: Function Retreives color (16bit) from picture stored in SD card.
* Input: offset of the Image in RES file, coordinates of desired pixel in the picture
* Output: Nothing
*******************************************************************************/
static unsigned int GetColor_FromSD(unsigned long Img, unsigned int xc, unsigned int yc){
unsigned int color_temp;
unsigned long readed;
unsigned long offset;
char *ptr;
int count;

  // set offset
  offset = Img + 6 + ((long)(800) * (yc - 1) + xc) * 2;
  count = 2;
  
  // retreive pointer to desired pixel
  ptr = MyTFT_Get_Data(offset,count,&readed);

  // read 16bit color value
  color_temp = (*ptr++);
  color_temp += (*ptr) << 8;
    
  return color_temp;
}

/*******************************************************************************
* Function GetColor_FromFlash(TImage *Img, unsigned int xc, unsigned int yc)
* ------------------------------------------------------------------------------
* Overview: Function Retreives color (16bit) from picture stored in flash.
* Input: Pointer of the Image in flash, coordinates of desired pixel in the picture
* Output: Nothing
*******************************************************************************/
static unsigned int GetColor_FromFlash(TImage *Img, unsigned int xc, unsigned int yc){
unsigned int color_temp;
const code char *pColor;
  pColor = Img->Picture_Name + 6;
  pColor = pColor + ((long)(Img->Width) * (yc - 1) + xc) * 2;
  color_temp = (*pColor++);
  color_temp += (*pColor) << 8;

  return color_temp;
}

/*******************************************************************************
* Function GetColor_FromSRAM(char *Image, unsigned int xc, unsigned int yc)
* ------------------------------------------------------------------------------
* Overview: Function Retreives color (16bit) from picture stored in SRAM.
* Input: Pointer of the Image in SRAM, coordinates of desired pixel in the picture
* Output: Nothing
*******************************************************************************/
static unsigned int GetColor_FromSRAM(char *Image, unsigned int xc, unsigned int yc){
unsigned int color_temp;
char *pColor;
  pColor = Image + 6;
  pColor = pColor + ((long)(800) * (yc - 1) + xc) * 2;
  color_temp = (*pColor++);
  color_temp += (*pColor) << 8;

  return color_temp;
}

/*******************************************************************************
* Function DisplayColor()
* ------------------------------------------------------------------------------
* Overview: Function Displays filled Circle in appropriate color on display
* Input: Nothing
* Output: Nothing
*******************************************************************************/
static void DisplayColor(){
  Circle1.Color = MyColors.color_16bit;
  DrawCircle(&Circle1);
}

/*******************************************************************************
* Function Display_color_text(TLabel *label, short val)
* ------------------------------------------------------------------------------
* Overview: Function Displays appropriate value in desired label
* Input: Pointer to Label, value
* Output: Nothing
*******************************************************************************/
static void Display_color_text(TLabel *label, short val) {
char text[7];
char text1[10];
char *ptr;
  ptr = label->Caption;
  TFTIntern_Set_Pen(RGBLight.Color,  0);
  TFTIntern_Set_Brush(1, RGBLight.Color, 0, 0, 0, 0);
  ShortToStr(val, text);
  strcpy(text1, ptr);
  strcat(text1, text);
  TFTIntern_Rectangle(label->Left, Label->Top, label->Left + 60, Label->Top + 30);
  label->Caption = text1;
  Drawlabel(label);
  label->Caption = ptr;
}

/*******************************************************************************
* Function Drive_RGB_Diode(TColors *colors)
* ------------------------------------------------------------------------------
* Overview: Function Outputs appropriate values to PWM outputs
* Input: Pointer to Colors struct
* Output: Nothing
*******************************************************************************/
static void Drive_RGB_Diode(TColors *colors){
  PWM_T1CCP0_Set_Duty((int)pwm_period1 / 32 * Colors->Red + 1, _PWM_INVERTED_DISABLE);
  PWM_T1CCP1_Set_Duty((int)pwm_period2 / 64 * colors->Green + 1, _PWM_INVERTED_DISABLE);
  PWM_T3CCP1_Set_Duty((int)pwm_period3 / 32 * Colors->Blue + 1, _PWM_INVERTED_DISABLE);
}

/*******************************************************************************
* Function RGB_OFF()
* ------------------------------------------------------------------------------
* Overview: Function turns off PWM modules, timers and turns off RGB diode
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void RGB_OFF(){
  PWM_T1CCP0_Stop();
  PWM_T1CCP1_Stop();
  PWM_T3CCP1_Stop();

  LED_R = 0;
  LED_G = 0;
  LED_B = 0;
}

/*******************************************************************************
* Function PickColor()
* ------------------------------------------------------------------------------
* Overview: Function reads coordinate of the touch and drives RGB accordingly
* Input: Nothing
* Output: Nothing
*******************************************************************************/
void PickColor(){
unsigned int mycolor;
  ycoo = Ycoord - 60;
  xcoo = Xcoord;
  if ((ycoo > 0) && (ycoo < 374)){
//    mycolor = GetColor_FromFlash(&Image4, xcoo, ycoo);     // read color  frm flash
//    mycolor = GetColor_FromSD(rgb_bitmap_bmp, xcoo, ycoo); // read color from Sd
    mycolor = GetColor_FromSRAM((char *)RGB_Color_Image_SRAM_ptr, xcoo, ycoo);
    
    // fill color struct
    MyColors.color_16bit = mycolor;
    MyColors.Red = mycolor >> 11;
    MyColors.Green = (mycolor >> 5) & 0x3F;
    MyColors.Blue = mycolor & 0x1F;
  
    // display color on TFT
    DisplayColor();
    Display_color_text(&Label11, MyColors.Red);
    Display_color_text(&Label12, MyColors.Green);
    Display_color_text(&Label13, MyColors.Blue);
    
    // Drive RGB diode
    Drive_RGB_Diode(&MyColors);
  }
}

/*******************************************************************************
* End of File
*******************************************************************************/