/*******************************************************************************
 * Project name:
     Omnibus example for mikromedia 5 for TIVA
 * Copyright:
     (c) Mikroelektronika, 2013.
 * Revision History:
     20131212:
       - initial release (JK);
 * Description:
     This project allow user to test various on board features of the
     mikromedia 5 for TIVA.
     
 * Test configuration:
     MCU:             TM4C129XNCZAD
                      http://www.ti.com/product/tm4c129xnczad
     dev.board:       mikromedia 5 for Tiva : ac:mikromedia_5_for_Tiva
                      http://www.mikroe.com/mikromedia/5/tiva/
     Oscillator:      HS-PLL 120.0000 MHz, 25.0000 MHz Crystal
     SW:              mikroC PRO for ARM
                      http://www.mikroe.com/mikroc/arm/
                      VisualTFT
                      http://www.mikroe.com/visualtft/
                      
 * NOTES:
     - VisualTFT uses following font for example
       Open sans which can be downloaded from here:
       http://www.fontsquirrel.com/fonts/open-sans
 
     - Accel example reads accel values of all three axes and displays them as
       sliding graph on TFT.
       
     - Paint example is great way to test touch panel and touch panel controller
       on the board. Simple draw lines and shapes over TFT. Board has TSC2006
       touch panel controller connected to SPI lines on microcontroller.
     
     - Tic Tac Toe game demonstration. Play simple game with your friend. User
       choose whether the game will be played on one board on two over RF
       communication. For this nRF24L01 is used
     
     - MP3 player example demonstrates usage of mp3 codec chip and SD card. Program
       uses FAT32 file system and appropriate library can be downloaded from libstock:
       http://www.libstock.com/projects/view/108/fat32-library
       User can navigate through folders and find desired mp3 or wav file and play it.
       
       IMPORTANT: FAT32 library supports only 8.3 file name format. So all folders and
         audio files have to be in short file name format.
         
     - Sensors example reads light sensor and if the light intensity is low it dims 
       background light. Also temperature Sensors displays measured temperature
       
     - RTC example is used to set Real Time Clock and Calendar. Board has back up
       battery and it will keep time when power is down. Also user can ajust time
       by setting simple roller wheel, or synhronize with GMT time by pressing sync
       Button. For this purpose CC3000 WiFi chip is used and our
       Network_WiFi library which is available on libstock:

       mikromedia connects via WiFi on first available Open Access Point.
       
     - Slide Show example demonstrates TFT. Images are soterd on SD card
       as resource file generated by VisualTFT. Scrolling is done by clicking on the
       buttons on the bottom of the screen.

     - RGB Light example demonstrates usage of RGB LED diode. It displays color
       palete on TFT and where ever user touch the Screen it converts it to RGB (5-6-5)
       and sets RGB diode in the same color.
       
     - USB Host : HID mouse can be used with this example. Connect it to USB comm port
       and use mouse to navigate through the example. ac:USB_MiniB_Plug_to_TypeA_Female_adapter
       should be used in this case. USB host library can be downloaded from libstock:
       http://www.libstock.com/projects/view/597/usb-host-library
       HID Mouse example is made to match specific Wireless mouse, 
       but it can be modified easily to work with any HID Mouse available (check HID_Mouse source)
       
*******************************************************************************/

#include "mikromedia_5_TIVA_objects.h"
#include "__Lib_nRF24L01.h"
#include "Init_routines.h"
#include "HID_Mouse.h"
#include "Tic_Tac_Toe.h"
#include "RTCC_driver.h"
#include "WiFi_main.h"

// external declarations
void Run_Example();

uint8_t USBHost_MSCApplication()
{

}

//******************************************************************************
// ISR
//******************************************************************************

//******************************************************************************
// Timer interrupt - Timer2A (16us)
// Used for nRF
//******************************************************************************
void Timer2A_interrupt() iv IVT_INT_TIMER2A_16_32_bit
{
  TIMER_ICR_TATOCINT_TIMER2_ICR_bit = 1;
  //Enter your code here
  NVIC_IntDisable(IVT_INT_TIMER2A_16_32_bit);
  TIMER2_IRQ();
  NVIC_IntDisable(IVT_INT_TIMER2A_16_32_bit);
}

//******************************************************************************
// External interrupt - INT GPIOG 2 pin
// Used for nRF
//******************************************************************************
void GPIOInt() iv IVT_INT_GPIOG 
{
  GPIO_PORTG_AHB_ICR = _GPIO_PINMASK_2;
  
  nRF24L01_IRQ();
}

//******************************************************************************
// Timer interrupt - Timer5 (1s)
// Used for TicTacToe
//******************************************************************************
void Timer3A_interrupt() iv IVT_INT_TIMER3A_16_32_bit 
{
  TIMER_ICR_TATOCINT_TIMER3_ICR_bit = 1;
  //Enter your code here
  TicTacToe_seconds_cnt ++;
  TTT_flag = 1;
}

//******************************************************************************
// External interrupt - INT GPIOB 2 pin
// Used for WiFi
//******************************************************************************
/*void GPIOInterrupt() iv IVT_INT_GPIOB ics ICS_AUTO
{
  Net_Wireless_CC3000_IntGpioHandler();
}*/

//******************************************************************************
// USB interrupt - USB HID Host
// Used for HID Mouse
//******************************************************************************
void USB0Interrupt() iv IVT_INT_USB
{
  USBHost_IntHandler();
}

//******************************************************************************
// Timer interrupt - USB HID Host
// Used for HID Mouse
//******************************************************************************
/*void Timer0A_interrupt() iv IVT_INT_TIMER0A_16_32_bit
{
  TIMER_ICR_TATOCINT_bit = 1;
  WiFi_Timer_ISR();
}*/

//******************************************************************************
// Main function
//******************************************************************************
void main() {
  Start_TP();

  Init_GPIO();
  Init_FAT();
  RTC_Init();
  HID_Mouse_Start();
  EnableInterrupts();

  while (1) {
    USBHost_Main();
    HID_Mouse_Handler();

    Check_TP();
    Run_Example();
  }
}